package cz.muni.fi.pa165;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.fi.pa165.movie.GenreEnum;
import cz.muni.fi.pa165.movie.ImageDto;
import cz.muni.fi.pa165.movie.MovieDto;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.List;

@SpringBootTest(classes = CoreApplication.class)
@AutoConfigureMockMvc
@ExtendWith(SpringExtension.class)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
public class MovieControllerTest {

    private final String MOVIES_URI = "/movies";

    private final MockMvc mockMvc;

    public MovieControllerTest(@Autowired MockMvc mockMvc) {
        this.mockMvc = mockMvc;
    }


    @Test
    @Order(1)
    public void testGetMovies() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get(MOVIES_URI))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string("[]"));

    }

    @Test
    @Order(2)
    public void testGetMovie() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get(MOVIES_URI + "/1"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string(""));

    }

    @Test
    @Order(3)
    public void testCreateMovie() throws Exception {
        var image = new ImageDto();
        image.setId("string");

        var movieDto = new MovieDto();
        movieDto.setId("string");
        movieDto.setTitle("string");
        movieDto.setDescription("string");
        movieDto.setDirector("string");
        movieDto.setActors(List.of("string"));
        movieDto.setGenres(List.of(GenreEnum.ACTION));
        movieDto.setImages(List.of(image));

        var objectMapper = new ObjectMapper();

        mockMvc.perform(MockMvcRequestBuilders.post(MOVIES_URI)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(movieDto))
                        .accept(MediaType.APPLICATION_JSON))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string(""));

    }

    @Test
    @Order(4)
    public void testEditMovie() throws Exception {
        var image = new ImageDto();
        image.setId("string");

        var movieDto = new MovieDto();
        movieDto.setId("string");
        movieDto.setTitle("string");
        movieDto.setDescription("string");
        movieDto.setDirector("string");
        movieDto.setActors(List.of("string"));
        movieDto.setGenres(List.of(GenreEnum.ACTION));
        movieDto.setImages(List.of(image));

        var objectMapper = new ObjectMapper();

        mockMvc.perform(MockMvcRequestBuilders.post(MOVIES_URI + "/1")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(movieDto))
                        .accept(MediaType.APPLICATION_JSON))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string(""));

    }
}