package cz.muni.fi.pa165;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.client.MockRestServiceServer;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.web.client.RestTemplate;

import java.net.URI;

import static org.springframework.test.web.client.match.MockRestRequestMatchers.method;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.requestTo;
import static org.springframework.test.web.client.response.MockRestResponseCreators.withStatus;

@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = LoginApplication.class)
@AutoConfigureMockMvc
@Import(AppConfig.class)
public class LoginControllerTest {

    private final String LOGIN_URI = "/login";

    private MockMvc mockMvc;
    private RestTemplate restTemplate;

    @Autowired
    public LoginControllerTest(RestTemplate restTemplate, MockMvc mockMvc) {
        this.restTemplate = restTemplate;
        this.mockMvc = mockMvc;
    }

    @Test
    public void testLogin() throws Exception {
        String username = "user123";
        String password = "pass456";
        String userControllerUrl = "http://localhost:8080/users/";
        String userDtoJson = "{\"username\":\"" + username + "\",\"password\":\"" + password + "\"}";

        MockRestServiceServer mockServer = MockRestServiceServer.createServer(restTemplate);
        mockServer.expect(requestTo(new URI(userControllerUrl + username)))
                .andExpect(method(HttpMethod.GET))
                .andRespond(withStatus(HttpStatus.OK)
                        .contentType(MediaType.APPLICATION_JSON)
                        .body(userDtoJson)
                );

        mockMvc.perform(MockMvcRequestBuilders.get(LOGIN_URI)
                        .param("username", username)
                        .param("password", password))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string("success"));

        mockServer.verify();
    }
}