package cz.muni.fi.pa156;

import cz.muni.fi.pa165.AppConfig;
import cz.muni.fi.pa165.RecommenderApplication;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Import;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.client.MockRestServiceServer;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;
import org.springframework.web.client.RestTemplate;

import static org.springframework.test.web.client.match.MockRestRequestMatchers.method;
import static org.springframework.test.web.client.match.MockRestRequestMatchers.requestTo;
import static org.springframework.test.web.client.response.MockRestResponseCreators.withSuccess;


@ExtendWith(SpringExtension.class)
@SpringBootTest(classes = RecommenderApplication.class)
@AutoConfigureMockMvc
@Import(AppConfig.class)
public class RecommenderControllerTest {

    private final String RECOMMENDER_URI = "/recommender";
    private MockMvc mockMvc;
    private RestTemplate restTemplate;

    @Autowired
    public RecommenderControllerTest(RestTemplate restTemplate, MockMvc mockMvc) {
        this.restTemplate = restTemplate;
        this.mockMvc = mockMvc;
    }

    @Test
    public void testGetSimilarMovies() throws Exception {
        String movieId = "1";
        String movieControllerUrl = "http://localhost:8080/movies";
        String movieDtosJson = "[{\"id\":\"1\",\"title\":\"Title\",\"description\":\"Description\",\"director\":\"Director\",\"actors\":null,\"genres\":null,\"images\":null}]";

        MockRestServiceServer mockServer = MockRestServiceServer.createServer(restTemplate);
        mockServer.expect(requestTo(movieControllerUrl))
                .andExpect(method(HttpMethod.GET))
                .andRespond(withSuccess(movieDtosJson, MediaType.APPLICATION_JSON));

        mockMvc.perform(MockMvcRequestBuilders.get(RECOMMENDER_URI + "/" + movieId))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().string(movieDtosJson));

        mockServer.verify();
    }
}
